package org.artifactory.addon.bridge;

import com.jfrog.retention.acl.bridge.TargetBundleRetentionBridgeToArtifactory;
import com.jfrog.retention.acl.exception.TargetBundleRetentionException;
import com.jfrog.retention.acl.model.ArtifactoryDetails;
import com.jfrog.sysconf.SysConfig;
import lombok.NonNull;
import lombok.extern.slf4j.Slf4j;
import org.jfrog.access.client.AccessClient;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.springframework.stereotype.Component;

/**
 * Empty implementation of the {@link TargetBundleRetentionBridgeToArtifactory} for Artifactory OSS.
 * <p>
 * The non-empty implementation resides in the Artifactory Pro, and has a higher priority
 * than this bean (in Artifactory Pro deployments this empty implementation is suppressed).
 */
@Slf4j
@Component
public class TargetBundleRetentionBridgeToArtifactoryEmptyImpl implements TargetBundleRetentionBridgeToArtifactory {

    private static final UnsupportedOperationException UNSUPPORTED_OPERATION =
            new UnsupportedOperationException("Target Bundle Retention add-on is available in Artifactory Pro");


    @Override
    public @NonNull ArtifactoryDetails getRunningVersion() {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public @NonNull DbType getDatabaseType() {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public @NonNull SysConfig getSysConfig() {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public @NonNull JdbcHelper getJdbcHelper() {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public @NonNull AccessClient getAccessClient() {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void deleteTargetBundle(@NonNull String bundleName, @NonNull String bundleVersion) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public boolean isTargetBundleMarkedAsKeep(@NonNull String bundleName, @NonNull String bundleVersion) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public boolean markTargetBundleKeep(@NonNull String bundleName, @NonNull String bundleVersion, boolean keep) {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public @NonNull String scheduleRetentionCleanup(@NonNull String cronExpression)
            throws TargetBundleRetentionException {
        throw UNSUPPORTED_OPERATION;
    }

    @Override
    public void cancelScheduledRetentionCleanup(@NonNull String taskToken) throws TargetBundleRetentionException {
        throw UNSUPPORTED_OPERATION;
    }
}
