package org.artifactory.queue.register;

import org.artifactory.event.queue.QueueCursorType;
import org.artifactory.event.queue.QueueService;
import org.artifactory.queue.services.PersistentQueue;
import org.artifactory.spring.ReloadableBean;
import org.artifactory.spring.RepoConfigChangeReloadableListener;

import java.util.Collection;
import java.util.Set;

/**
 * Queue API Registration interface used by the other Queue services
 *
 * @author Avishay Halpren
 */
public interface PersistentQueueRegistrationService
        extends QueueService, ReloadableBean, RepoConfigChangeReloadableListener {

    /**
     * Registration of a repoKey:
     * - creates queues for the specific repo
     */
    void registerRepository(String repoKey);

    /**
     * Removes registration of a repoKey:
     * - removes and stop queues for the specific repo
     * - cancel triggered jobs
     * - deletes the marker of the queue
     */
    void deRegisterRepository(String repoKey);

    Collection<PersistentQueue> getQueues(String repoKey, QueueCursorType cursorType);

    Collection<PersistentQueue> getQueues(Set<String> reposKey);

    Collection<PersistentQueue> getAllQueues();

}
