package org.artifactory.repo.service.validation;

import lombok.experimental.UtilityClass;
import org.artifactory.descriptor.config.CentralConfigDescriptor;

import static org.artifactory.repo.service.validation.RepositoryNameValidator.isBlacklistedPrefix;
import static org.artifactory.repo.service.validation.RepositoryNameValidator.isForbiddenCharacter;
import static org.artifactory.repo.service.validation.RepositoryNameValidator.isForbiddenName;
import static org.artifactory.repo.service.validation.RepositoryNameValidator.isKeyAvailableInCentralConfigDescriptor;
import static org.artifactory.repo.service.validation.RepositoryNameValidator.isKeyBlank;
import static org.artifactory.repo.service.validation.RepositoryNameValidator.isMaxLengthExceed;
import static org.artifactory.repo.service.validation.RepositoryNameValidator.isReservedName;

@UtilityClass
public class RepositoryNameYamlConfigurationValidator {

    public static void validateRepoName(String repoKey, String repoType,
            CentralConfigDescriptor centralConfigDescriptor)
            throws IllegalArgumentException {
        isKeyBlank(repoKey);
        isMaxLengthExceed(repoKey, repoType);
        isReservedName(repoKey);
        isBlacklistedPrefix(repoKey);
        isForbiddenName(repoKey);
        isForbiddenCharacter(repoKey);
        isKeyAvailableInCentralConfigDescriptor(repoKey, centralConfigDescriptor);
    }
}
