/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.build;

import org.artifactory.api.build.BuildService;
import org.artifactory.api.context.ArtifactoryContext;
import org.artifactory.api.context.ArtifactoryContextThreadBinder;
import org.artifactory.test.ArtifactoryHomeBoundTest;
import org.jfrog.build.api.Build;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.Assert;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

import java.util.Date;

import static java.lang.String.valueOf;
import static org.artifactory.repo.system.SystemRepos.BUILDINFO_REPO_KEY;
import static org.jfrog.build.api.Build.formatBuildStarted;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.verify;

/**
 * @author jbaruch
 * @since 04/05/12
 */
public class DetailedBuildRunTests extends ArtifactoryHomeBoundTest {

    @Mock
    private ArtifactoryContext artifactoryContext;
    @Mock
    private BuildService buildService;

    private DetailedBuildRunImpl original;

    @BeforeMethod
    public void setup() {
        MockitoAnnotations.initMocks(this);
        Build build = new Build();
        build.setNumber(valueOf(42));
        build.setName("test-build");
        build.setStarted(formatBuildStarted(new Date().getTime()));
        original = new DetailedBuildRunImpl(build, BUILDINFO_REPO_KEY);

        doReturn(buildService).when(artifactoryContext).beanForType(BuildService.class);
        doReturn(BUILDINFO_REPO_KEY).when(buildService).getBuildInfoRepoForDefaultProject();

        ArtifactoryContextThreadBinder.bind(artifactoryContext);
    }

    @AfterMethod
    public void tearDown() {
        ArtifactoryContextThreadBinder.unbind();
    }

    @Test
    public void testCopy() {
        DetailedBuildRun copy = original.copy();
        Assert.assertNotEquals(copy, original);//started should be different
        Assert.assertEquals(copy.getName(), original.getName());
        Assert.assertEquals(copy.getNumber(), original.getNumber());
        Assert.assertNotEquals(copy.getStarted(), original.getStarted());
        verify(buildService).getBuildInfoRepoForDefaultProject();
    }

    // TODO[tr]: enable
    @Test(enabled = false)
    public void testCopyWithNewNumber() {
        String newNumber = "42-r";
        DetailedBuildRun copy = original.copy(newNumber);
        Assert.assertNotEquals(copy, original);//started should be different
        Assert.assertEquals(copy.getName(), original.getName());
        Assert.assertNotEquals(copy.getNumber(), original.getNumber());
        Assert.assertEquals(copy.getNumber(), newNumber);
        Assert.assertNotEquals(copy.getStarted(), original.getStarted());
        verify(buildService).getBuildInfoRepoForDefaultProject();
    }
}
