package org.artifactory.metrics.framework.providers.util;

import org.jfrog.metrics.model.Metric;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertEquals;

class ProviderUtilTest {
    @ParameterizedTest
    @MethodSource("poolNameDataProvider")
    void testAddMetricWithPoolLabel(String poolName, String expectedMetricName) {
        List<Metric> metrics = new ArrayList<>();
        Map<String, Object> labels = Map.of("pool", poolName);
        String metricName = "metric_name";
        String description = "description";
        Number metricValue = 1;
        ProviderUtil.addMetricWithLabelsToList(metrics, labels, metricName, Metric.MetricType.COUNTER, description,
                metricValue);
        assertEquals(expectedMetricName, metrics.get(0).getName());
    }

    @Test
    void testAddMetricWithoutPoolLabel() {
        List<Metric> metrics = new ArrayList<>();
        Map<String, Object> labels = Map.of();
        String metricName = "metric_name";
        String description = "description";
        Number metricValue = 1;
        ProviderUtil.addMetricWithLabelsToList(metrics, labels, metricName, Metric.MetricType.COUNTER, description,
                metricValue);
        assertEquals("metric_name", metrics.get(0).getName());
    }

    private static Stream<Arguments> poolNameDataProvider() {
        return Stream.of(
                Arguments.of("", "metric_name"),
                Arguments.of("docker_io", "metric_name_docker_io"),
                Arguments.of("docker.io", "metric_name_docker_io"),
                Arguments.of("some.pool.p", "metric_name_some_pool_p"),
                Arguments.of("docker-i-o", "metric_name_docker_i_o"),
                Arguments.of("docker#i.o", "metric_name_docker_i_o")
        );
    }
}