package org.artifactory.ph;

import org.artifactory.common.repo.PackageType;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.service.PackageResourceServiceAdapter;
import org.artifactory.core.api.ph.service.PackageSpecificationAdapter;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;

import javax.servlet.http.HttpServletRequest;

import static org.junit.jupiter.api.Assertions.assertNull;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static org.mockito.MockitoAnnotations.openMocks;

/**
 * @author Volodymyr Shulga
 */
class PackageResourceServiceImplTest {

    private PackageResourceServiceImpl packageResourceService;

    @Mock
    private PackageSpecificationAdapter packageSpecificationAdapter;
    @Mock
    private PackageResourceServiceAdapter packageResourceServiceAdapter;
    @Mock
    private HttpServletRequest request;

    private AutoCloseable closeable;

    @BeforeEach
    void setup() {
        closeable = openMocks(this);
        packageResourceService = new PackageResourceServiceImpl(packageResourceServiceAdapter,
                packageSpecificationAdapter);
    }

    @AfterEach
    void destroy() throws Exception {
        closeable.close();
    }

    @Test
    void getVersionedResourceV3() throws PackageException {
        String repoKey = "repo";
        String version = "v3";
        assertNull(packageResourceService.getVersionedSubResource(PackageType.NUGET, version, repoKey, request));

        when(packageSpecificationAdapter.isSupportedPackage(PackageType.NUGET)).thenReturn(true);
        packageResourceService.getVersionedSubResource(PackageType.NUGET, version, repoKey, request);
        verify(packageResourceServiceAdapter).getVersionedSubResource(PackageType.NUGET, version, repoKey, request);
    }

    @Test
    void getVersionedResourceV2() throws PackageException {
        String repoKey = "repo";
        String version = null;
        assertNull(packageResourceService.getVersionedSubResource(PackageType.NUGET, version, repoKey, request));
        when(packageSpecificationAdapter.isSupportedPackage(PackageType.NUGET)).thenReturn(true);

        packageResourceService.getVersionedSubResource(PackageType.NUGET, version, repoKey, request);
        verify(packageResourceServiceAdapter).getVersionedSubResource(PackageType.NUGET, version, repoKey, request);
    }

    @Test
    void reindex() throws PackageException {
        String repoKey = "repo";
        assertNull(packageResourceService.reindex(PackageType.NUGET, repoKey));

        when(packageSpecificationAdapter.isSupportedPackage(PackageType.NUGET)).thenReturn(true);
        packageResourceService.reindex(PackageType.NUGET, repoKey);
        verify(packageResourceServiceAdapter).reindex(PackageType.NUGET, repoKey);
    }

    @Test
    void getNonVersionedSubResource() throws PackageException {
        String repoKey = "repo";
        assertNull(packageResourceService.getNonVersionedSubResource(PackageType.NUGET, repoKey, request));
        when(packageSpecificationAdapter.isSupportedPackage(PackageType.NUGET)).thenReturn(true);

        packageResourceService.getNonVersionedSubResource(PackageType.NUGET, repoKey, request);
        verify(packageResourceServiceAdapter).getNonVersionedSubResource(PackageType.NUGET, repoKey, request);
    }

    @Test
    void getPackageInfo() throws PackageException {
        String repoKey = "repo";
        String path = "dir/package";
        assertNull(packageResourceService.getPackageInfo(repoKey, path));
        when(packageSpecificationAdapter.isSupportedPackage(repoKey)).thenReturn(true);

        packageResourceService.getPackageInfo(repoKey, path);
        verify(packageResourceServiceAdapter).getPackageInfo(repoKey, path);
    }

    @Test
    void getVersionNativeReadme() throws PackageException {
        String path = "repo/dir/package";
        assertNull(packageResourceService.getVersionNativeReadme(path));

        verify(packageResourceServiceAdapter).getVersionNativeReadme(path);
    }
}