package org.artifactory.logging.layout;

import ch.qos.logback.classic.spi.ILoggingEvent;
import org.artifactory.logging.entry.AccessEntryPart;
import org.artifactory.logging.entry.SaasBaseResponseEntryPart;
import org.artifactory.logging.entry.SaasOutboundRequestEntryPart;
import org.artifactory.logging.entry.SaasRequestEntryPart;
import org.artifactory.logging.entry.ServiceEntryAppPart;
import org.jfrog.access.common.ServiceType;
import org.joda.time.format.DateTimeFormat;
import org.joda.time.format.DateTimeFormatter;
import org.slf4j.MDC;

import java.util.Map;

/**
 * @author Tomer Slutsky
 */
public class SaasLogParts {
    private static final DateTimeFormatter DATE_TIME_FORMAT =
            DateTimeFormat.forPattern("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").withZoneUTC();

    public static ServiceEntryAppPart getServiceEntryAppPart(Map<String, Object> data, ILoggingEvent event) {
        return ServiceEntryAppPart.builder().dateTime(DATE_TIME_FORMAT.print(
                        Long.parseLong(data.get("timestamp").toString())))
                .service(ServiceType.ARTIFACTORY)
                .logLevel(data.get("level").toString())
                .traceId(event.getMDCPropertyMap().get("uber-trace-id"))
                .clazz(event.getCallerData()[0].getFileName())
                .line(String.valueOf(event.getCallerData()[0].getLineNumber()))
                .thread(event.getThreadName())
                .message(data.get("message").toString()).build();
    }

    public static AccessEntryPart getAccessEntryPart(Map<String, Object> data, ILoggingEvent event) {
        Object[] argumentArray = event.getArgumentArray();
        return AccessEntryPart.builder()
                .dateTime(DATE_TIME_FORMAT.print(Long.parseLong(data.get("timestamp").toString())))
                .traceId(event.getMDCPropertyMap().get("uber-trace-id"))
                .actionResponse(argumentArray[0] + " " + argumentArray[1])
                .repositoryPath(argumentArray[2].toString())
                .message(argumentArray[3].toString())
                .username(argumentArray[4].toString())
                .userIp(argumentArray[5].toString())
                .build();
    }

    public static SaasRequestEntryPart getRequestEntryPart(ILoggingEvent event) {
        Object[] argumentArray = event.getArgumentArray();
        return SaasRequestEntryPart.builder()
                .traceId(MDC.get("uber-trace-id"))
                .client(argumentArray[2].toString())
                .user(argumentArray[3].toString())
                .method(argumentArray[4].toString())
                .path(argumentArray[5].toString())
                .length(Long.parseLong(argumentArray[7].toString())).userAgent(argumentArray[10].toString()).build();
    }

    public static SaasBaseResponseEntryPart getResponseEntryPart(ILoggingEvent event) {
        Object[] argumentArray = event.getArgumentArray();
        return SaasBaseResponseEntryPart.builder().dateTime(argumentArray[0].toString())
                .length(Long.parseLong(argumentArray[8].toString()))
                .status(argumentArray[6].toString())
                .duration(Long.parseLong(argumentArray[9].toString())).build();
    }

    public static SaasOutboundRequestEntryPart getOutboundRequestEntryPart(ILoggingEvent event) {
        Object[] argumentArray = event.getArgumentArray();
        return SaasOutboundRequestEntryPart.builder()
                .traceId(MDC.get("uber-trace-id"))
                .remoteRepository(argumentArray[2].toString())
                .user(argumentArray[3].toString())
                .method(argumentArray[4].toString())
                .path(argumentArray[5].toString())
                .length(Long.parseLong(argumentArray[7].toString())).build();
    }
}
