package org.artifactory.repo.interceptor;

import org.artifactory.repo.RepoConfigChange;
import org.artifactory.repo.RepoDetailsType;
import org.artifactory.spring.RepoConfigChangeReloadableListener;

import java.util.List;
import java.util.Map;

/**
 * @author Tamar Rosen
 */
public interface RepoConfigChangeInterceptor {

    /**
     * Performs an on before interception on the provided repository config changes.
     * Depending on certain conditions, might mutate the provided map by adding, updating or removing a
     * {@link RepoConfigChange}.
     *
     * @param mutableRepoConfigChanges A mutable map of repository configuration changes including the action, old
     *                                 and new config, grouped by repo type.
     */
    default void onBeforeSave(Map<RepoDetailsType, List<RepoConfigChange>> mutableRepoConfigChanges) {

    }

    /**
     * Called after all {@link RepoConfigChangeReloadableListener} have been reloaded.
     *
     * @param repoConfigChanges Map of repository configuration changes including the action, old and new config,
     *                          grouped by repo type.
     */
    default void onAfterSave(Map<RepoDetailsType, List<RepoConfigChange>> repoConfigChanges) {

    }

    default void setDeleteAllMode(boolean deleteAll) {

    }
}
