package org.artifactory.model;

import lombok.Data;
import lombok.NoArgsConstructor;
import org.artifactory.common.config.metadata.MetadataInfo;
import org.artifactory.common.config.metadata.MetadataVersion;
import org.jfrog.common.config.validation.NotUnicode;

/**
 * This class contains base and common configurations which are relevant for any repository type
 * (Local/Remote/Virtual/Federated)
 *
 * @author Tamar Rosen
 */
@Data
@NoArgsConstructor
@MetadataVersion(metadataVersion = 1)
public class BaseConfig {
    @MetadataInfo(defaultValue = "0")
    private long modelVersion;
    @NotUnicode
    @MetadataInfo
    private String description;
    @NotUnicode
    @MetadataInfo
    private String notes;
    @NotUnicode
    @MetadataInfo(skip = true)
    private String repoLayoutRef; // ex-xml-idref
    @NotUnicode
    @MetadataInfo(defaultValue = "**/*")
    private String includesPattern = "**/*";
    @NotUnicode
    @MetadataInfo
    private String excludesPattern;

    BaseConfig(BaseConfig baseConfig) {
        this.modelVersion = baseConfig.modelVersion;
        this.description = baseConfig.getDescription();
        this.notes = baseConfig.getNotes();
        this.repoLayoutRef = baseConfig.getRepoLayoutRef();
        this.includesPattern = baseConfig.getIncludesPattern();
        this.excludesPattern = baseConfig.getExcludesPattern();
    }
}
