package org.artifactory.event.queue;

import java.util.List;


/**
 * A functional interface of the persistent queue framework.<br>
 * Must be implemented - will contain the main applicative logic to be executed.<br>
 * <br>
 * e.g: Incremental-index of a repo, replication etc...
 *
 * @author avishayh
 */
@FunctionalInterface
public interface QueueEventImplementor {

    /**
     * implement your application logic here.
     *
     * @param events - the unhandled (filtered,aggregated,sorted) events until now
     * @param key    - repoKey, or the group key supplied by QueueEventAggregator (if used)
     * @throws QueueEventException - you can wrap your exceptions with this one to allow the advancement of the cursor
     */
    void execute(List<? extends QueueEvent> events, String key) throws QueueEventException;

}
