package org.artifactory.core.api.ph.listener;


import lombok.NonNull;
import org.artifactory.core.api.ph.exception.PackageException;

/**
 * Manage the listeners for RT config changes provided by the
 * {@link org.artifactory.ph.PackageConfigurationChangesInterceptorImpl}
 *
 * The Implementation is provided by the package handler (in OSS, a default empty bean is loaded by the core)
 */
public interface PackageConfigChangedEventListener {

    /**
     * Respond to global config changes in RT (e.g BASE_URL)
     *
     * @param packageConfigChangedEventType - The changed config
     */
    void onGlobalConfigChange(@NonNull PackageConfigChangedEventType packageConfigChangedEventType);

    /**
     * Respond to repository config changes in RT(E.g: REPO_CREATE)
     * @param context Contains the information needed to the package specific implementation
     * @throws PackageException
     */
    void onRepoConfigChange(@NonNull PackageRepoConfigListenerContext context) throws PackageException;

}
