package org.artifactory.core.api.ph.service;

import org.artifactory.descriptor.repo.RepoType;

/**
 * Service for collecting specific API interactions.
 * e.g. docker pull\push\login API calls.
 *
 * @author BarakH
 */
public interface PackageAnalyticsService {

    /**
     * This method should be called when a successful package download process was accomplished.
     * e.g. when a docker manifest.json was downloaded.
     *
     * @param packageName - Full package name in case of a multipart name.
     */
    default void onSuccessfulPackageDownload(RepoType repoType, String repoKey, String packageName, String version) {
    }

    /**
     * This method should be called when a successful package download process was accomplished.
     * e.g. when a docker manifest.json was uploaded.
     *
     * @param packageName - Full package name in case of a multipart name.
     */
    default void onSuccessfulPackageUpload(RepoType repoType, String repoKey, String packageName, String version) {
    }

    /**
     * This method should be called when a user successfully generated a token.
     * e.g. when a user accomplished docker login process.
     */
    default void userCreatedToken(RepoType repoType, String userName) {
    }
}
