package org.artifactory.core.api.ph.service;

import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.fs.ItemInfo;
import org.artifactory.md.Properties;
import org.artifactory.repo.RepoPath;

import java.util.EnumSet;
import java.util.Set;

/**
 * A service to trigger direct package indexing in the Metadata service.
 * <p>
 * IMPORTANT NOTE
 * 1. When implementing a new package manager, consider upload, copy, move and replication events (Deletion in already
 * handled).
 * 2. Add your package type to {@link  PackageMetadataServiceIndexHandler#REALTIME_INDEXING_PACKAGES}
 * </p>
 *
 * @author BarakH
 */
public interface PackageMetadataServiceIndexHandler {

    /**
     * Supported package types by this indexer.
     * The rest of the package types, will get indexed by the old event-based mechanism.
     */
    Set<RepoType> REALTIME_INDEXING_PACKAGES =
            EnumSet.of(RepoType.Docker, RepoType.Npm, RepoType.Gradle, RepoType.Maven, RepoType.SBT, RepoType.Ivy,
                    RepoType.P2);

    /**
     * Create an async deletion task for the given repo path.
     */
    void deletePackageAsync(RepoPath repoPath);

    /**
     * Create an async deletion task for the given repo path.
     */
    void deletePackageSync(RepoPath repoPath);

    /**
     * Create an async task to create or update a package.
     *
     * @param itemInfo   - Of the lead file of the package artifact.ll
     * @param properties - An updated set of properties (after extraction).
     */
    void indexPackageAsync(ItemInfo itemInfo, Properties properties);


    /**
     * Create or update a package in Metadata service.
     *
     * @param itemInfo   - Of the lead file of the package artifact.
     * @param properties - An updated set of properties (after extraction).
     */
    void indexPackageSync(ItemInfo itemInfo, Properties properties);

}
