package org.artifactory.core.api.ph.service;

import lombok.NonNull;
import org.artifactory.common.resource.BaseResource;
import org.artifactory.core.api.ph.exception.PackageException;
import org.artifactory.core.api.ph.virtual.resolution.PackageUploadContext;

import javax.ws.rs.core.Response;
import java.io.File;
import java.io.InputStream;

/**
 * API that Artifactory provides for the package handler layer for deploy/upload artifacts
 */
public interface PackageUploadService {

    /**
     * Upload an artifact to local/remote cache or leading local in virtual repository
     *
     * @param repoKey     Repository key
     * @param path        Artifact path
     * @param inputStream Input stream
     * @throws PackageException in case od missing permission missing repository or internal error
     */
    void upload(String repoKey, String path, InputStream inputStream) throws PackageException;

    /**
     * Upload an artifact to local or leading local in virtual repository.
     * The upload context allows uploading artifacts with matrix params and extra headers.
     *
     * @param uploadContext - Upload context (repoKey, path, inputStream, properties, headers...)
     * @return Response of the actual upload process
     * @throws PackageException in case od missing permission missing repository or internal error
     */
    Response upload(PackageUploadContext uploadContext) throws PackageException;

    /**
     * This is INTERNAL method, it should be used only in the package handler framework and should not be exposed
     * to the various package modules
     *
     * @param repoKey     the virtual\remote cache repo to upload
     * @param path        the path to upload
     * @param inputStream the artifact content
     * @return file resource
     * @throws PackageException in case of repository not found or any internal error
     */
    BaseResource uploadToVirtualOrRemoteCache(String repoKey, String path, InputStream inputStream) throws
            PackageException;

    /**
     * Proxies the call to ArtifactoryHome.getTempUploadDir
     *
     * @return writable dir for temp uploads.
     */
    @NonNull
    File getTempUploadDir();

}
