package org.artifactory.storage.db;

import lombok.NonNull;
import org.jfrog.common.BinaryStoreConfigService;
import org.jfrog.common.BinaryStoreConstValues;
import org.jfrog.storage.binstore.db.dao.BinariesDao;
import org.jfrog.storage.binstore.db.dao.RemoteBinaryTaskDao;
import org.jfrog.storage.binstore.db.services.BinariesDbServiceImpl;
import org.jfrog.storage.binstore.service.BinariesDbService;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

/**
 * @author TamirH
 */
@Configuration
public class BinariesStoreBeansConfig {

    @Bean
    public BinariesDao createBinariesDao(@NonNull BinaryStoreConfigService binaryStoreConfigService) {
        return BinariesDao.builder()
                .binaryProviderPruneChunkSizeInt(
                        binaryStoreConfigService.getIntlValue(BinaryStoreConstValues.binaryProviderPruneChunkSize))
                .gcBinariesToDeleteBatchSizeLong(
                        binaryStoreConfigService.getLongValue(BinaryStoreConstValues.gcBinariesToDeleteBatchSize))
                .dbOperationsBulkSizeInt(
                        binaryStoreConfigService.getIntlValue(BinaryStoreConstValues.dbOperationsBulkSize))
                .dbType(binaryStoreConfigService.getDbType())
                .gcSkipOrderByFullGcBoolean(
                        binaryStoreConfigService.getBoolValue(BinaryStoreConstValues.gcSkipOrderByFullGc))
                .jdbcHelper(binaryStoreConfigService.getJdbcHelper())
                .build();
    }

    @Bean
    public RemoteBinaryTaskDao createRemoteBinaryTaskDao(@NonNull BinaryStoreConfigService binaryStoreConfigService) {
        return new RemoteBinaryTaskDao(binaryStoreConfigService.getJdbcHelper(), binaryStoreConfigService.getDbType());
    }

    @Bean
    public BinariesDbService createBinariesDbService(@NonNull BinariesDao binariesDao) {
        return new BinariesDbServiceImpl(binariesDao);
    }

}
