package org.artifactory.storage.db;

import com.jfrog.sysconf.SysConfig;
import io.opentracing.Tracer;
import org.apache.http.impl.client.CloseableHttpClient;
import org.artifactory.addon.AddonType;
import org.artifactory.addon.AddonsManager;
import org.artifactory.common.ConstantValues;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.entitlement.EntitlementType;
import org.artifactory.opentracing.ArtifactoryTracer;
import org.artifactory.util.HttpClientConfigurator;
import org.jfrog.client.http.model.ProxyConfig;
import org.jfrog.common.BinaryStoreConfigService;
import org.jfrog.common.BinaryStoreConstValues;
import org.jfrog.storage.DbType;
import org.jfrog.storage.JdbcHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEvent;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.io.File;

import static org.apache.commons.lang.StringUtils.isNotBlank;

/**
 * @author TamirH
 */
@Service
public class BinaryStoreConfigServiceImpl implements BinaryStoreConfigService {

    private JdbcHelper jdbcHelper;
    private AddonsManager addonsManager;
    private ArtifactoryTracer artifactoryTracer;
    private ApplicationEventPublisher eventPublisher;
    @Autowired
    @Lazy
    public void setAddonsManager(AddonsManager addonsManager) {
        this.addonsManager = addonsManager;
    }

    @Autowired
    public void setJdbcHelper(JdbcHelper jdbcHelper) {
        this.jdbcHelper = jdbcHelper;
    }

    @Override
    public JdbcHelper getJdbcHelper() {
        return this.jdbcHelper;
    }

    @Override
    public DbType getDbType() {
        return ArtifactoryHome.get().getDBProperties().getDbType();
    }

    public Tracer getTracer() {
        return artifactoryTracer.getTracer();
    }

    @Autowired
    @Lazy
    public void setArtifactoryTracer(ArtifactoryTracer artifactoryTracer) {
        this.artifactoryTracer = artifactoryTracer;
    }

    @Autowired
    public void setEventPublisher(ApplicationEventPublisher eventPublisher) {
        this.eventPublisher = eventPublisher;
    }

    @Override
    public File getArtifactoryDataDir() {
        return ArtifactoryHome.get().getDataDir();
    }

    @Override
    public boolean isAol() {
        return addonsManager.isAddonSupported(AddonType.AOL);
    }

    @Override
    public boolean isLicenseInstalled() {
        return addonsManager.isLicenseInstalled();
    }

    @Override
    public boolean isEntitlementEnforced() {
        return addonsManager.isEntitlementEnforced();
    }

    @Override
    public boolean isEntitledForRshProvider() {
        return addonsManager.isEntitled(EntitlementType.BINARY_PROVIDER_RSH);
    }

    @Override
    public boolean isEntitledForCustomEndpoint() {
        return addonsManager.isEntitled(EntitlementType.BINARY_PROVIDER_CUSTOM_ENDPOINT);
    }

    @Override
    public String getArtifactoryRunningModeName() {
        return addonsManager.getArtifactoryRunningMode().name();
    }

    @Override
    public boolean isTrialLicense() {
        return addonsManager.isTrialLicense();
    }

    @Override
    public boolean isJCR() {
        return addonsManager.getArtifactoryRunningMode().isJCR();
    }

    @Override
    public SysConfig getSysConfig() {
        return ArtifactoryHome.get().getSysConfig();
    }

    @Override
    public File getBinaryStoreGcpCredentialsFile() {
        return ArtifactoryHome.get().getBinaryStoreGcpCredentialsFile();
    }

    @Override
    public CloseableHttpClient createClosableHttpClient(Integer maxConnectionsPerRoute,
            Integer maxConnectionsTotal, Integer connectionTimeout,
            ProxyConfig proxy, boolean withParams) {
        HttpClientConfigurator configurator = new HttpClientConfigurator();
        if (maxConnectionsPerRoute != null) {
            configurator.maxConnectionsPerRoute(maxConnectionsPerRoute);
        }
        if (maxConnectionsTotal != null) {
            configurator.maxTotalConnections(maxConnectionsTotal);
        }
        if (connectionTimeout != null) {
            configurator.connectionTimeout(connectionTimeout);
        }
        if (proxy != null && isNotBlank(proxy.getHost())) {
            configurator.proxy(proxy);
        }
        if (withParams) {
            return configurator.build(true);
        }
        return configurator.build();
    }

    @Override
    public boolean isHaConfigured() {
        return ArtifactoryHome.get().isHaConfigured();
    }

    @Override
    public int getIntlValue(BinaryStoreConstValues value) {
        return ConstantValues.valueOf(value.getPropertyName()).getInt();
    }

    @Override
    public boolean getBoolValue(BinaryStoreConstValues value) {
        return ConstantValues.valueOf(value.getPropertyName()).getBoolean();
    }

    @Override
    public long getLongValue(BinaryStoreConstValues value) {
        return ConstantValues.valueOf(value.getPropertyName()).getLong();
    }

    @Override
    public void publishStorageEvent(ApplicationEvent event) {
        eventPublisher.publishEvent(event);
    }
}
