package org.artifactory.storage.db.build.dao.utils;

import org.apache.commons.lang.StringUtils;
import org.artifactory.api.rest.build.ContinueBuildFilter;
import org.artifactory.build.BuildId;
import org.artifactory.storage.db.build.entity.BuildIdEntity;
import org.mapstruct.ap.internal.util.Strings;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import static org.artifactory.api.rest.build.ContinueBuildFilter.OrderBy.BUILD_NAME;
import static org.artifactory.api.rest.common.model.continues.util.Direction.ASC;

public class BuildsDaoUtils {

    private BuildsDaoUtils() {

    }

    public static String createWhereClauseFromContinueBuildFilter(ContinueBuildFilter continueBuildFilter,
            List<Object> sqlParametersList, String alias) {
        BuildId continueBuild = continueBuildFilter.getContinueBuildId();
        List<String> whereClauses = new ArrayList<>();
        String aliasPrefix = StringUtils.isNotBlank(alias) ? alias + "." : "";
        if (continueBuild != null) {
            if (StringUtils.isNotBlank(continueBuild.getBuildRepo())) {
                whereClauses.add(String.format("(%srepo = ?)", aliasPrefix));
                sqlParametersList.add(continueBuild.getBuildRepo());
            }
            Object continueValue = getContinueValue(continueBuildFilter);
            char directionChar = continueBuildFilter.getDirection() == ASC ? '>' : '<';
            if (continueBuildFilter.getOrderBy() == BUILD_NAME) {
                whereClauses.add(String.format("%sbuild_name %s ?", aliasPrefix, directionChar));
                sqlParametersList.add(continueBuild.getName());
            } else {
                whereClauses.add(String.format("(%s%s %s ? OR (%s%s = ? AND %sbuild_name %s ?))",
                        aliasPrefix, continueBuildFilter.getOrderByStr(), directionChar, aliasPrefix,
                        continueBuildFilter.getOrderByStr(), aliasPrefix, directionChar));
                sqlParametersList.add(continueValue);
                sqlParametersList.add(continueValue);
                sqlParametersList.add(continueBuild.getName());
            }
        }
        if (Objects.isNull(continueBuild) || StringUtils.isBlank(continueBuild.getBuildRepo())) {
            String buildRepo = continueBuildFilter.getBuildRepo();
            if (StringUtils.isNotBlank(buildRepo)) {
                whereClauses.add(String.format("(%srepo = ?)", aliasPrefix));
                sqlParametersList.add(buildRepo);
            }
        }
        if (StringUtils.isNotBlank(continueBuildFilter.getSearchStr())) {
            whereClauses.add(String.format("(%sbuild_name like ?)", aliasPrefix));
            sqlParametersList.add("%" + continueBuildFilter.getSearchStr() + "%");
        }

        String separator = " AND ";
        return Strings.join(whereClauses, separator);
    }

    public static List<BuildIdEntity> populateBuildIdEntryList(ResultSet rs) throws SQLException {
        List<BuildIdEntity> buildIdEntities = new ArrayList<>();
        while (rs.next()) {
            BuildIdEntity buildId =
                    new BuildIdEntity(rs.getLong("build_id"), rs.getString("build_name"), rs.getString("build_number"),
                            rs.getLong("build_date"), rs.getString("repo"));
            buildIdEntities.add(buildId);
        }
        return buildIdEntities;
    }

    public static String createOrderByStr(ContinueBuildFilter continueBuildFilter) {
        String orderByStr;
        if (continueBuildFilter.getOrderBy() == BUILD_NAME) {
            orderByStr = String.format("build_name %s", continueBuildFilter.getDirection());
        } else {
            orderByStr = String.format("%s %s, build_name %s", continueBuildFilter.getOrderByStr(),
                    continueBuildFilter.getDirection(), continueBuildFilter.getDirection());
        }
        return orderByStr;
    }

    private static Object getContinueValue(ContinueBuildFilter continueBuildFilter) {

        BuildId buildId = continueBuildFilter.getContinueBuildId();
        switch (continueBuildFilter.getOrderBy()) {
            case BUILD_NAME:
                return buildId.getName();
            case BUILD_NUMBER:
                return buildId.getNumber();
            case BUILD_DATE:
            default:
                return buildId.getStartedDate().getTime();
        }
    }

}
