package org.artifactory.storage.db.repo.service;

import org.artifactory.storage.db.repo.entity.DbRepositoryConfig;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @author Tamar Rosen
 */
public interface RepositoryConfigStorageService {
    /**
     * Find a stored repository configuration by its key
     *
     * @param repoKey the key of the repository
     * @return the repository config DB model
     */
    DbRepositoryConfig findRepositoryByKey(String repoKey);

    /**
     * Find all stored repositories
     *
     * @return List of repository config DB models
     */
    List<DbRepositoryConfig> getAllRepositories();

    /**
     * Find all stored repositories by their keys
     *
     * @return List of repository config DB models
     */
    Set<DbRepositoryConfig> findRepositoryConfigsByKeys(Set<String> repoKeys);

    /**
     * Insert a new repository configuration
     *
     * @param dbRepositoryConfig repository configuration to insert
     */
    void insertRepositoryConfig(DbRepositoryConfig dbRepositoryConfig);

    /**
     * Insert multiple repository configurations
     *
     * @param dbRepositoryConfigs a list of repository config DB models to insert
     */
    @Transactional
    void bulkInsertRepositoryConfigs(Set<DbRepositoryConfig> dbRepositoryConfigs);

    /**
     * Delete multiple repository configurations
     *
     * @param repoKeys a list of repository keys to delete
     */
    @Transactional
    void bulkDeleteRepositoryConfigs(Set<String> repoKeys);

    /**
     * Update an existing repository configuration
     *
     * @param dbRepositoryConfig the updated repository configuration
     */
    void updateRepositoryConfig(DbRepositoryConfig dbRepositoryConfig);

    @Transactional
    void bulkUpdateRepositoryConfigs(Set<DbRepositoryConfig> dbRepositoryConfigs);

    void delete(String repoKey);

    boolean exists(String repoKey);

    int countAll();

    int deleteAll();

    Map<String, Long> getAllRepositoryKeysAndModificationDates();

    void flushBulkChanges(Set<DbRepositoryConfig> repositoriesToCreate, Set<DbRepositoryConfig> repositoriesToUpdate,
            Set<String> repositoriesToDelete);

    Collection<DbRepositoryConfig> findRepositoriesByDetailsType(String repoDetailsType);
}