package org.artifactory.storage.db.build.itest.artifactbuilds;

import org.artifactory.addon.build.BuildAddon;
import org.artifactory.storage.db.build.aql.BuildsAql;
import org.artifactory.storage.db.build.dao.BuildArtifactsDao;
import org.artifactory.storage.db.build.dao.BuildDependenciesDao;
import org.artifactory.storage.db.build.dao.BuildModulesDao;
import org.artifactory.storage.db.build.dao.BuildsDao;
import org.artifactory.storage.db.build.entity.BuildArtifact;
import org.artifactory.storage.db.build.entity.BuildDependency;
import org.artifactory.storage.db.build.entity.BuildEntity;
import org.artifactory.storage.db.build.entity.BuildModule;
import org.artifactory.storage.db.fs.dao.NodesDao;
import org.artifactory.storage.db.fs.entity.Node;
import org.artifactory.storage.db.itest.DbBaseTest;
import org.jfrog.storage.binstore.db.dao.BinariesDao;
import org.jfrog.storage.binstore.db.entity.BinaryEntity;
import org.joda.time.DateTime;
import org.springframework.beans.factory.annotation.Autowired;

import java.sql.SQLException;
import java.util.HashSet;

public abstract class ArtifactBuildsTestBase extends DbBaseTest {

    @Autowired
    BuildsAql buildsAql;

    @Autowired
    BuildDependenciesDao buildDependenciesDao;

    @Autowired
    BuildsDao buildsDao;

    @Autowired
    BuildModulesDao buildModulesDao;

    @Autowired
    NodesDao nodesDao;

    @Autowired
    BinariesDao binariesDao;

    @Autowired
    BuildArtifactsDao buildArtifactsDao;

    protected void createArtifact(Node node) throws SQLException {
        binariesDao.create(new BinaryEntity(node.getSha1Actual(), node.getSha2(), node.getMd5Actual(), 0));
        nodesDao.create(node);
    }

    protected void createBuildModule(BuildModule buildModule) throws SQLException {
        buildModule.setProperties(new HashSet<>());
        buildModulesDao.createBuildModule(buildModule);
    }

    protected void createBuild(int buildId, String buildName) throws SQLException {
        long buildTime = DateTime.now().getMillis();
        buildsDao.createBuild(new BuildEntity(buildId,
                buildName,
                "1", buildTime, "some-ci-url.com", buildTime,
                "test", buildTime, "test", BuildAddon.BUILD_INFO_REPO_NAME));
    }

    protected void linkModuleToArtifact(BuildModule buildModule, Node node) throws SQLException {
        buildArtifactsDao.createBuildArtifact(
                new BuildArtifact(node.getNodeId(), buildModule.getModuleId(), node.getName(), "file",
                        node.getSha1Actual(), node.getMd5Actual(), ""));
    }

    protected void linkModuleToDependantArtifact(int dependencyId, BuildModule buildModule, Node node)
            throws SQLException {
        buildDependenciesDao.createBuildDependency(
                new BuildDependency(dependencyId, buildModule.getModuleId(), node.getName(), "", "file",
                        node.getSha1Actual(), node.getMd5Actual(), ""));
    }

}
