package org.artifactory.storage.db.upgrades.itest.version;

import lombok.NonNull;
import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.jfrog.converter.DbVersion;
import org.jfrog.converter.model.ChangeLogContext;
import org.jfrog.storage.util.DbUtils;
import org.testng.annotations.Test;

import java.io.IOException;
import java.sql.SQLException;
import java.util.List;

import static org.artifactory.storage.db.event.dao.NodeEventsCursorDao.EVENTS_CURSOR_TABLE;
import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v126;
import static org.artifactory.storage.db.version.ArtifactoryDBVersion.v127;
import static org.jfrog.storage.util.DbUtils.withConnection;
import static org.testng.AssertJUnit.assertFalse;
import static org.testng.AssertJUnit.assertTrue;

public class V126DropDefaultColumnInCursorTableTest extends UpgradeBaseTest {


    @Test
    public void testNoDefault() throws SQLException, IOException {
        resetToVersion(ArtifactoryDBVersion.v125);
        assertTrue(isDefaultExist("status"));
        assertTrue(isDefaultExist("last_updated"));

        final List<ArtifactoryDBVersion> dbVersions = List.of(v126, v127);
        for (DbVersion dbVersion : dbVersions) {
            try (ChangeLogContext context = new ChangeLogContext(jdbcHelper.getDataSource().getConnection(),
                    dbProperties.getDbType(), dbVersion)) {
                dbVersion.converter().convert(context);
            }
        }

        assertFalse(isDefaultExist("last_updated"));
        assertFalse(isDefaultExist("status"));
    }

    private boolean isDefaultExist(@NonNull String column) throws SQLException {
        return withConnection(jdbcHelper, conn -> DbUtils
                .isColumnHasDefaultValue(conn.getMetaData(), dbProperties.getDbType(), EVENTS_CURSOR_TABLE,
                        column));
    }

    @Override
    protected ArtifactoryDBVersion upToVersion() {
        return v127;
    }
}


