/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.ui.rest.service.admin.configuration.repositories.util.builder;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.artifactory.addon.p2.P2Repo;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.api.config.RepositoryConfigService;
import org.artifactory.common.crypto.CryptoHelper;
import org.artifactory.common.home.ArtifactoryHome;
import org.artifactory.descriptor.config.CentralConfigDescriptor;
import org.artifactory.descriptor.config.MutableCentralConfigDescriptor;
import org.artifactory.descriptor.property.PropertySet;
import org.artifactory.descriptor.repo.KeyPairDescriptor;
import org.artifactory.descriptor.repo.ProxyDescriptor;
import org.artifactory.descriptor.repo.RepoType;
import org.artifactory.descriptor.repo.ReverseProxyDescriptor;
import org.artifactory.descriptor.repo.ReverseProxyRepoConfig;
import org.artifactory.model.FederatedRepoConfig;
import org.artifactory.model.LocalConfig;
import org.artifactory.model.LocalRepoConfig;
import org.artifactory.model.ProxyRepoConfig;
import org.artifactory.model.RealRepoConfig;
import org.artifactory.model.ReleaseBundleRepoConfig;
import org.artifactory.model.RemoteRepoConfig;
import org.artifactory.model.RepoConfig;
import org.artifactory.model.VirtualRepoConfig;
import org.artifactory.model.XrayRepoConfig;
import org.artifactory.repo.CdnRedirectRepoConfig;
import org.artifactory.repo.DownloadRedirectRepoConfig;
import org.artifactory.repo.federated.FederatedMember;
import org.artifactory.repo.packagetype.DockerApiVersion;
import org.artifactory.repo.virtual.VirtualCacheConfig;
import org.artifactory.ui.rest.model.admin.configuration.propertysets.PropertySetNameModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.AdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.BasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.CdnRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.DownloadRedirectRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.ProxyAwareConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.RepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepoAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.FederatedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.federation.MirrorMemberConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.CommonLocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.local.LocalRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.releasebundles.ReleaseBundlesRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteCacheRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteNetworkRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.remote.RemoteRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.AlpineTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.BowerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CargoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ChefTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CocoaPodsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ComposerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.ConanTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CondaTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.CranTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DebTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.DockerTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GemsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GenericTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GitLfsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.GoTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.HelmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.MavenTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NpmTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.NugetTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.P2TypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.PypiTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.SwiftTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TerraformTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.TypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.VcsTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.typespecific.YumTypeSpecificConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualAdvancedRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualBasicRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualRepositoryConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.repository.virtual.VirtualSelectedRepository;
import org.artifactory.ui.rest.model.admin.configuration.repository.xray.XrayRepoConfigModel;
import org.artifactory.ui.rest.model.admin.configuration.reverseProxy.ReverseProxyRepoModel;
import org.artifactory.util.AolUtils;
import org.jfrog.client.util.PathUtils;
import org.jfrog.common.JsonUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Stream;

import static java.util.stream.Collectors.toList;
import static org.artifactory.repo.RepoDetailsType.FEDERATED;
import static org.artifactory.repo.RepoDetailsType.LOCAL;
import static org.artifactory.util.PackageTypeConfigFields.BLOCK_PUSHING_SCHEMA_1;
import static org.artifactory.util.PackageTypeConfigFields.BOWER_REGISTRY_URL;
import static org.artifactory.util.PackageTypeConfigFields.CACHING_LOCAL_FOREIGN_LAYERS_ENABLED;
import static org.artifactory.util.PackageTypeConfigFields.CALCULATE_YUM_METADATA;
import static org.artifactory.util.PackageTypeConfigFields.CARGO_ANONYMOUS_ACCESS;
import static org.artifactory.util.PackageTypeConfigFields.CARGO_INTERNAL_INDEX;
import static org.artifactory.util.PackageTypeConfigFields.CHARTS_BASE_URL;
import static org.artifactory.util.PackageTypeConfigFields.COCOA_PODS_SPECS_URL;
import static org.artifactory.util.PackageTypeConfigFields.COMPOSER_REGISTRY_URL;
import static org.artifactory.util.PackageTypeConfigFields.DDEB_SUPPORTED;
import static org.artifactory.util.PackageTypeConfigFields.DEBIAN_DEFAULT_ARCHITECTURES;
import static org.artifactory.util.PackageTypeConfigFields.DEBIAN_OPTIONAL_INDEX_COMPRESSION_FORMATS;
import static org.artifactory.util.PackageTypeConfigFields.DEBIAN_TRIVIAL_LAYOUT;
import static org.artifactory.util.PackageTypeConfigFields.DOCKER_API_VERSION;
import static org.artifactory.util.PackageTypeConfigFields.DOCKER_TAG_RETENTION;
import static org.artifactory.util.PackageTypeConfigFields.DOWNLOAD_CONTEXT_PATH;
import static org.artifactory.util.PackageTypeConfigFields.ENABLE_COMPOSER_V1_INDEXING;
import static org.artifactory.util.PackageTypeConfigFields.ENABLE_FILE_LISTS_INDEXING;
import static org.artifactory.util.PackageTypeConfigFields.EXTERNAL_DEPENDENCIES_ENABLED;
import static org.artifactory.util.PackageTypeConfigFields.EXTERNAL_DEPENDENCIES_PATTERNS;
import static org.artifactory.util.PackageTypeConfigFields.EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF;
import static org.artifactory.util.PackageTypeConfigFields.FEED_CONTEXT_PATH;
import static org.artifactory.util.PackageTypeConfigFields.FETCH_JARS_EAGERLY;
import static org.artifactory.util.PackageTypeConfigFields.FETCH_SOURCES_EAGERLY;
import static org.artifactory.util.PackageTypeConfigFields.FORCE_CONAN_AUTHENTICATION;
import static org.artifactory.util.PackageTypeConfigFields.FORCE_MAVEN_AUTHENTICATION;
import static org.artifactory.util.PackageTypeConfigFields.FORCE_NUGET_AUTHENTICATION;
import static org.artifactory.util.PackageTypeConfigFields.GIT_REGISTRY_URL;
import static org.artifactory.util.PackageTypeConfigFields.MAX_UNIQUE_TAGS;
import static org.artifactory.util.PackageTypeConfigFields.OPTIONAL_INDEX_COMPRESSION_FORMATS;
import static org.artifactory.util.PackageTypeConfigFields.P2_ORIGINAL_URL;
import static org.artifactory.util.PackageTypeConfigFields.P2_URLS;
import static org.artifactory.util.PackageTypeConfigFields.POM_REPOSITORY_REFERENCES_CLEANUP_POLICY;
import static org.artifactory.util.PackageTypeConfigFields.PYPI_REGISTRY_URL;
import static org.artifactory.util.PackageTypeConfigFields.PYPI_REPOSITORY_SUFFIX;
import static org.artifactory.util.PackageTypeConfigFields.REJECT_INVALID_JARS;
import static org.artifactory.util.PackageTypeConfigFields.RESOLVE_DOCKER_TAGS_BY_TIMESTAMP;
import static org.artifactory.util.PackageTypeConfigFields.SUPPRESS_POM_CONSISTENCY_CHECKS;
import static org.artifactory.util.PackageTypeConfigFields.SYMBOL_SERVER_URL;
import static org.artifactory.util.PackageTypeConfigFields.TERRAFORM_PROVIDERS_URL;
import static org.artifactory.util.PackageTypeConfigFields.TERRAFORM_REGISTRY_URL;
import static org.artifactory.util.PackageTypeConfigFields.TERRAFORM_TYPE;
import static org.artifactory.util.PackageTypeConfigFields.USE_NAMESPACES;
import static org.artifactory.util.PackageTypeConfigFields.V3_FEED_URL;
import static org.artifactory.util.PackageTypeConfigFields.VCS_GIT_DOWNLOAD_URL;
import static org.artifactory.util.PackageTypeConfigFields.VCS_GIT_PROVIDER;
import static org.artifactory.util.PackageTypeConfigFields.VCS_TYPE;
import static org.artifactory.util.PackageTypeConfigFields.YUM_GROUP_FILENAMES;
import static org.artifactory.util.PackageTypeConfigFields.YUM_ROOT_DEPTH;

/**
 * Utility class for converting model to repo config
 *
 * @author Dan Feldman
 */
@Slf4j
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class RepoConfigDescriptorBuilder {
    private CentralConfigService centralConfig;
    private RepositoryConfigService repositoryConfigService;

    static String getUrlWithoutSubpath(String url) {
        int slashslash = url.indexOf("//") + 2;
        int nextSlash = url.indexOf('/', slashslash);
        return nextSlash < 0 ? url : PathUtils.trimSlashes(url.substring(0, nextSlash));
    }

    @Autowired
    public void setCentralConfig(CentralConfigService centralConfig) {
        this.centralConfig = centralConfig;
    }

    @Autowired
    public void setRepositoryConfigService(RepositoryConfigService repositoryConfigService) {
        this.repositoryConfigService = repositoryConfigService;
    }

    public LocalConfig buildLocalConfig(LocalRepositoryConfigModel model) {
        LocalConfig localConfig =
                LocalRepoConfig.create(model.getGeneral().getRepoKey(), model.getTypeSpecific().getRepoType());
        populateSharedBasicConfigValues(model.getBasic(), localConfig);
        buildAndSetXrayConfig(localConfig, model.getBasic());
        buildAndSetCdnRedirectConfig(localConfig, model);
        buildAndSetDownloadRedirectConfig(localConfig, model.getAdvanced());
        populateSharedAdvancedDescriptorValues(model.getAdvanced(), localConfig);
        populateSharedTypeSpecificDescriptorValues(model.getTypeSpecific(), localConfig);
        populateLocalTypeSpecificDescriptorValues(model.getTypeSpecific(), localConfig);
        setDefaultGpgKeyIfNeeded(localConfig);
        return localConfig;
    }

    public FederatedRepoConfig buildFederatedRepoConfig(FederatedRepositoryConfigModel model) {
        FederatedRepoConfig federatedRepoConfig =
                FederatedRepoConfig.create(model.getGeneral().getRepoKey(), model.getTypeSpecific().getRepoType());
        populateSharedBasicConfigValues(model.getBasic(), federatedRepoConfig);
        populateFederationValues(model, federatedRepoConfig);
        buildAndSetXrayConfig(federatedRepoConfig, model.getBasic());
        buildAndSetCdnRedirectConfig(federatedRepoConfig, model);
        buildAndSetDownloadRedirectConfig(federatedRepoConfig, model.getAdvanced());
        populateSharedAdvancedDescriptorValues(model.getAdvanced(), federatedRepoConfig);
        populateSharedTypeSpecificDescriptorValues(model.getTypeSpecific(), federatedRepoConfig);
        populateFederatedRepoAdvancedDescriptorValues(model.getAdvanced(), federatedRepoConfig);
        populateLocalTypeSpecificDescriptorValues(model.getTypeSpecific(), federatedRepoConfig);
        setDefaultGpgKeyIfNeeded(federatedRepoConfig);
        return federatedRepoConfig;
    }

    public RemoteRepoConfig buildRemoteRepoConfig(RemoteRepositoryConfigModel model) {
        RemoteRepoConfig remoteRepoConfig =
                RemoteRepoConfig.create(model.getGeneral().getRepoKey(), model.getTypeSpecific().getRepoType());
        populateSharedBasicConfigValues(model.getBasic(), remoteRepoConfig);
        populateRemoteBasicRepoConfigValues(model.getBasic(), remoteRepoConfig);
        buildAndSetXrayConfig(remoteRepoConfig, model.getBasic());
        buildAndSetCdnRedirectConfig(remoteRepoConfig, model);
        buildAndSetDownloadRedirectConfig(remoteRepoConfig, model.getAdvanced());
        populateSharedAdvancedDescriptorValues(model.getAdvanced(), remoteRepoConfig);
        populateRemoteAdvancedDescriptorValues(model.getAdvanced(), remoteRepoConfig,
                model.getTypeSpecific().getRepoType());
        populateSharedTypeSpecificDescriptorValues(model.getTypeSpecific(), remoteRepoConfig);
        populateRemoteTypeSpecificDescriptorValues(model.getTypeSpecific(), remoteRepoConfig);
        return remoteRepoConfig;
    }

    public VirtualRepoConfig buildVirtualRepoConfig(VirtualRepositoryConfigModel model) {
        VirtualRepoConfig virtualRepoConfig =
                VirtualRepoConfig.create(model.getGeneral().getRepoKey(), model.getTypeSpecific()
                        .getRepoType());
        populateSharedBasicConfigValues(model.getBasic(), virtualRepoConfig);
        // if not validate resolve virtual values from config descriptor (instead of config service)
        populateVirtualBasicConfigValues(model.getBasic(), virtualRepoConfig);
        populateVirtualAdvancedConfigValues(model.getAdvanced(), virtualRepoConfig);
        populateVirtualTypeSpecific(model.getTypeSpecific(), virtualRepoConfig);
        setDefaultGpgKeyIfNeeded(virtualRepoConfig);
        return virtualRepoConfig;
    }

    public ReleaseBundleRepoConfig buildReleaseBundleRepoConfig(ReleaseBundlesRepositoryConfigModel model) {
        ReleaseBundleRepoConfig releaseBundleRepoConfig =
                ReleaseBundleRepoConfig.create(model.getGeneral().getRepoKey());
        populateSharedBasicConfigValues(model.getBasic(), releaseBundleRepoConfig);
        buildAndSetXrayConfig(releaseBundleRepoConfig, model.getBasic());
        populateSharedAdvancedDescriptorValues(model.getAdvanced(), releaseBundleRepoConfig);
        populateSharedTypeSpecificDescriptorValues(model.getTypeSpecific(), releaseBundleRepoConfig);
        populateLocalTypeSpecificDescriptorValues(model.getTypeSpecific(), releaseBundleRepoConfig);
        return releaseBundleRepoConfig;
    }

    public ReverseProxyDescriptor buildReverseProxyDescriptor(AdvancedRepositoryConfigModel model,
            RepoConfig repoConfig) {
        ReverseProxyRepoModel reverseProxy = model.getReverseProxy();
        CentralConfigDescriptor mutableDescriptor = centralConfig.getMutableDescriptor();
        ReverseProxyDescriptor reverseProxyDescriptor = mutableDescriptor.getCurrentReverseProxy();
        if (reverseProxy != null && reverseProxyDescriptor != null) {
            if (reverseProxy.getServerPort() == null) {
                reverseProxyDescriptor.deleteReverseProxyConfig(repoConfig.getKey());
            } else {
                ReverseProxyRepoConfig reverseProxyRepoConfig = new ReverseProxyRepoConfig();
                reverseProxyRepoConfig.setServerName(reverseProxy.getServerName());
                reverseProxyRepoConfig.setPort(reverseProxy.getServerPort());
                reverseProxyRepoConfig.setRepoRef(repoConfig.getKey());
                reverseProxyDescriptor.addReverseProxyRepoConfig(reverseProxyRepoConfig);
            }
        }
        return reverseProxyDescriptor;
    }

    void populateVirtualBasicConfigValues(VirtualBasicRepositoryConfigModel model,
            VirtualRepoConfig virtualRepoConfig) {
        virtualRepoConfig.getRepoTypeConfig().setRepositoryRefs(model.getSelectedRepositories().stream()
                .map(VirtualSelectedRepository::getRepoName)
                .filter(repoName -> repositoryConfigService.exists(repoName))
                .filter(Objects::nonNull)
                .toList());
        if (StringUtils.isNotBlank(model.getDefaultDeploymentRepo())) {
            Stream.of(repositoryConfigService.findRepositoriesByType(LOCAL),
                            repositoryConfigService.findRepositoriesByType(FEDERATED))
                    .flatMap(Collection::stream)
                    .filter(repoConfig -> repoConfig.getKey().equals(model.getDefaultDeploymentRepo()))
                    .map(LocalConfig.class::cast)
                    .findAny()
                    .ifPresent(repoConfig -> virtualRepoConfig.getRepoTypeConfig().setDefaultDeploymentRepoRef(
                            repoConfig.getKey()));

        }
    }

    private void populateFederationValues(CommonLocalRepositoryConfigModel model,
            FederatedRepoConfig federatedRepoConfig) {
        List<MirrorMemberConfigModel> mirrorMembers = ((FederatedRepositoryConfigModel) model)
                .getFederation().getMirrors();
        if (mirrorMembers != null) {
            federatedRepoConfig.getRepoTypeConfig().setFederatedMembers(
                    mirrorMembers.stream().map(this::toFederatedMember).collect(toList()));
        }
    }

    private FederatedMember toFederatedMember(MirrorMemberConfigModel mirrorMemberConfigModel) {
        FederatedMember federatedMember = new FederatedMember();
        federatedMember.setUrl(mirrorMemberConfigModel.getUrl());
        //federatedMember.setMode(mirrorMemberConfigModel.getMode()); future support
        federatedMember.setEnabled(mirrorMemberConfigModel.isEnabled());
        return federatedMember;
    }

    private void populateFederatedRepoAdvancedDescriptorValues(FederatedRepoAdvancedRepositoryConfigModel advanced,
            FederatedRepoConfig federatedRepoConfig) {
        populateProxyValues(advanced, federatedRepoConfig);
    }

    /**
     * populate config model's proxy values to repository config
     *
     * Note: default proxy must not be set as reference but to be handled as part of system config change event
     * mechanism
     *
     * @see org.artifactory.repo.confighandler.ProxyConfigChangeEventHandler
     */
    private void populateProxyValues(ProxyAwareConfigModel model, ProxyRepoConfig proxyRepoConfig) {
        boolean isDisabledProxy = Boolean.TRUE.equals(model.isDisableProxy());
        proxyRepoConfig.setDisableProxy(isDisabledProxy);
        if (isDisabledProxy) {
            return;
        }
        String proxyRef = model.getProxy();
        if (StringUtils.isNotBlank(proxyRef)) {
            ProxyDescriptor proxyDescriptor = centralConfig.getDescriptor().getProxy(proxyRef);
            if (proxyDescriptor == null) {
                log.debug("Proxy '{}' does not exist in the config descriptor", proxyRef);
            } else if (!proxyDescriptor.isPlatformDefault()) {
                proxyRepoConfig.setProxyRef(proxyRef);
            }
        }
    }

    /**
     * Populates basic descriptor values that are shared between local and remote repos
     */
    private void populateSharedBasicConfigValues(BasicRepositoryConfigModel model,
            RepoConfig repoConfig) {
        MutableCentralConfigDescriptor mutableDescriptor = centralConfig.getMutableDescriptor();
        repoConfig.getBaseConfig().setDescription(model.getPublicDescription());
        repoConfig.getBaseConfig().setNotes(model.getInternalDescription());
        repoConfig.getBaseConfig().setIncludesPattern(model.getIncludesPattern());
        repoConfig.getBaseConfig().setExcludesPattern(model.getExcludesPattern());
        if (StringUtils.isNotBlank(model.getLayout())) { //Don't enforce on virtual
            repoConfig.getBaseConfig()
                    .setRepoLayoutRef(mutableDescriptor.getRepoLayout(model.getLayout()).getName());
        }
        KeyPairDescriptor keyPair = mutableDescriptor
                .getKeyPair(model.getKeypair());
        KeyPairDescriptor primaryKeyPair = mutableDescriptor.getKeyPair(model.getPrimaryKeyPair());
        if (StringUtils.isNotBlank(model.getKeypair()) && keyPair != null) {
            repoConfig.getSecurityConfig().setPrimaryKeyPairRef(keyPair.getKey());
        } else if (StringUtils.isNotBlank(model.getPrimaryKeyPair()) && primaryKeyPair != null) {
            repoConfig.getSecurityConfig().setPrimaryKeyPairRef(primaryKeyPair.getKey());
        } else {
            repoConfig.getSecurityConfig().setPrimaryKeyPairRef(null);
        }
        KeyPairDescriptor secondaryKeyPair = mutableDescriptor
                .getKeyPair(model.getSecondaryKeyPair());
        if (StringUtils.isNotBlank(model.getSecondaryKeyPair()) && secondaryKeyPair != null) {
            repoConfig.getSecurityConfig().setSecondaryKeyPairRef(secondaryKeyPair.getKey());
        } else {
            repoConfig.getSecurityConfig().setSecondaryKeyPairRef(null);
        }
    }

    private void setDefaultGpgKeyIfNeeded(RepoConfig localConfig) {
        if (KeyPairDescriptor.GPG_REPO_TYPES.contains(localConfig.getPackageType())
                && Objects.isNull(localConfig.getSecurityConfig().getPrimaryKeyPairRef())) {
            KeyPairDescriptor keyPair = centralConfig.getDescriptor().getKeyPair(KeyPairDescriptor.DEFAULT_GPG_KEY);
            if (Objects.nonNull(keyPair)) {
                localConfig.getSecurityConfig().setPrimaryKeyPairRef(keyPair.getKey());
            }
        }
    }

    /**
     * Populates remote basic repo config values
     */
    private void populateRemoteBasicRepoConfigValues(RemoteBasicRepositoryConfigModel model,
            RemoteRepoConfig remoteRepoConfig) {
        remoteRepoConfig.getRepoTypeConfig().setUrl(model.getUrl());
        if (model.getRemoteLayoutMapping() != null) {
            remoteRepoConfig.getRepoTypeConfig()
                    .setRemoteRepoLayoutRef(
                            centralConfig.getDescriptor().getRepoLayout(model.getRemoteLayoutMapping()).getName());
        }
        remoteRepoConfig.getRepoTypeConfig().setOffline(model.isOffline());
        remoteRepoConfig.getRepoTypeConfig().setContentSynchronisation(model.getContentSynchronisation());
    }

    private void populateVirtualAdvancedConfigValues(VirtualAdvancedRepositoryConfigModel model,
            VirtualRepoConfig virtualRepoConfig) {
        virtualRepoConfig.getRepoTypeConfig()
                .setArtifactoryRequestsCanRetrieveRemoteArtifacts(model.getRetrieveRemoteArtifacts());
        virtualRepoConfig.getSecurityConfig().setSignedUrlTtl(model.getSignedUrlTtl());
        buildReverseProxyDescriptor(model, virtualRepoConfig);
        virtualRepoConfig.getSecurityConfig().setHideUnauthorizedResources(model.getHideUnauthorizedResources());
    }

    /**
     * Populates advanced repo config values that are shared between local and remote repos
     */
    private void populateSharedAdvancedDescriptorValues(AdvancedRepositoryConfigModel model,
            RealRepoConfig localConfig) {
        localConfig.getRepoTypeConfig().setBlackedOut(model.isBlackedOut());
        localConfig.getRepoTypeConfig().setArchiveBrowsingEnabled(model.getAllowContentBrowsing());
        localConfig.getSecurityConfig().setSignedUrlTtl(model.getSignedUrlTtl());
        List<PropertySetNameModel> propertySetNameModels = model.getPropertySets();
        if (propertySetNameModels != null) {
            List<PropertySet> propertySets = model.getPropertySets().stream()
                    .map(propSet -> getPropSetByName(propSet.getName()))
                    .filter(Objects::nonNull)
                    .toList();
            localConfig.getRepoTypeConfig().setPropertySetRefs(propertySets.stream().map(PropertySet::getName).collect(
                    toList()));
        }
        buildReverseProxyDescriptor(model, localConfig);
        localConfig.getRepoTypeConfig().setPriorityResolution(Boolean.TRUE.equals(model.getPriorityResolution()));
    }

    /**
     * Populates remote advanced repo config values
     */
    private void populateRemoteAdvancedDescriptorValues(RemoteAdvancedRepositoryConfigModel model,
            RemoteRepoConfig remoteRepoConfig, RepoType repoType) {
        //network
        RemoteNetworkRepositoryConfigModel network = model.getNetwork();
        if (network != null) {
            populateProxyValues(network, remoteRepoConfig);
            remoteRepoConfig.getRepoTypeConfig().setLocalAddress(network.getLocalAddress());
            remoteRepoConfig.getRepoTypeConfig().setUsername(network.getUsername());
            remoteRepoConfig.getRepoTypeConfig()
                    .setPassword(CryptoHelper.encryptIfNeeded(ArtifactoryHome.get(), network.getPassword()));
            remoteRepoConfig.getRepoTypeConfig().setSocketTimeoutMillis(network.getSocketTimeout());
            remoteRepoConfig.getRepoTypeConfig().setAllowAnyHostAuth(network.getLenientHostAuth());
            remoteRepoConfig.getRepoTypeConfig().setEnableCookieManagement(network.getCookieManagement());
            remoteRepoConfig.getRepoTypeConfig().setClientTlsCertificate(network.getSelectedInstalledCertificate());
        }
        //cache
        RemoteCacheRepositoryConfigModel cache = model.getCache();
        if (cache != null) {
            remoteRepoConfig.getRepoTypeConfig()
                    .setUnusedArtifactsCleanupPeriodHours(cache.getKeepUnusedArtifactsHours());
            remoteRepoConfig.getRepoTypeConfig().setRetrievalCachePeriodSecs(cache.getRetrievalCachePeriodSecs());
            remoteRepoConfig.getRepoTypeConfig().setAssumedOfflinePeriodSecs(cache.getAssumedOfflineLimitSecs());
            remoteRepoConfig.getRepoTypeConfig()
                    .setMissedRetrievalCachePeriodSecs(cache.getMissedRetrievalCachePeriodSecs());
            remoteRepoConfig.getRepoTypeConfig()
                    .setMetadataRetrievalTimeoutSecs(cache.getMetadataRetrievalTimeoutSecs());
        }
        //other
        remoteRepoConfig.getRepoTypeConfig().setQueryParams(model.getQueryParams());
        remoteRepoConfig.getRepoTypeConfig().setPropagateQueryParams(model.isPropagateQueryParams());
        remoteRepoConfig.getRepoTypeConfig().setHardFail(model.getHardFail());
        remoteRepoConfig.getRepoTypeConfig().setSynchronizeProperties(model.getSynchronizeArtifactProperties());
        remoteRepoConfig.getRepoTypeConfig().setShareConfiguration(model.isShareConfiguration());
        remoteRepoConfig.getRepoTypeConfig().setBlockMismatchingMimeTypes(model.isBlockMismatchingMimeTypes());
        remoteRepoConfig.getRepoTypeConfig().setMismatchingMimeTypesOverrideList(
                PathUtils.collectionToDelimitedString(model.getMismatchingMimeTypesOverrideList()));
        remoteRepoConfig.getRepoTypeConfig().setBypassHeadRequests(model.isBypassHeadRequests());
        remoteRepoConfig.getRepoTypeConfig().setDisableUrlNormalization(model.isDisableUrlNormalization());

        if (repoType.equals(RepoType.Docker) && Boolean.FALSE.equals(model.isStoreArtifactsLocally())) {
            throw new IllegalArgumentException("Docker prerequisite - artifacts must be stored locally in cache");
        } else {
            remoteRepoConfig.getRepoTypeConfig().setStoreArtifactsLocally(model.isStoreArtifactsLocally());
        }
    }

    /**
     * Populates type specific values shared by local and remote repos
     */
    private void populateSharedTypeSpecificDescriptorValues(TypeSpecificConfigModel type,
            RealRepoConfig localConfig) {
        switch (type.getRepoType()) {
            case Maven, Gradle, Ivy, SBT -> {
                MavenTypeSpecificConfigModel maven = (MavenTypeSpecificConfigModel) type;
                localConfig.getRepoTypeConfig().setMaxUniqueSnapshots(maven.getMaxUniqueSnapshots());
                localConfig.getRepoTypeConfig().setHandleReleases(maven.getHandleReleases());
                localConfig.getRepoTypeConfig().setHandleSnapshots(maven.getHandleSnapshots());
                localConfig.getPackageTypeConfig()
                        .put(SUPPRESS_POM_CONSISTENCY_CHECKS.getKey(),
                                String.valueOf(maven.getSuppressPomConsistencyChecks()));
            }
            case NuGet -> localConfig.getPackageTypeConfig().put(FORCE_NUGET_AUTHENTICATION.getKey(),
                    String.valueOf(((NugetTypeSpecificConfigModel) type).isForceNugetAuthentication()));
            case Conan -> localConfig.getPackageTypeConfig().put(FORCE_CONAN_AUTHENTICATION.getKey(),
                    String.valueOf(((ConanTypeSpecificConfigModel) type).isForceConanAuthentication()));
            default -> {
            }
        }
    }

    /**
     * Populates type specific values for local repos
     */
    private void populateLocalTypeSpecificDescriptorValues(TypeSpecificConfigModel type,
            LocalConfig localConfig) {
        final Map<String, String> packageTypeConfig = localConfig.getPackageTypeConfig();
        switch (type.getRepoType()) {
            case Maven, Gradle, Ivy, SBT -> {
                MavenTypeSpecificConfigModel maven = (MavenTypeSpecificConfigModel) type;
                localConfig.getRepoTypeConfig().setSnapshotVersionBehavior(maven.getSnapshotVersionBehavior());
                localConfig.getRepoTypeConfig().setChecksumPolicyType(maven.getLocalChecksumPolicy());
            }
            case YUM -> {
                YumTypeSpecificConfigModel yum = (YumTypeSpecificConfigModel) type;
                packageTypeConfig
                        .put(YUM_GROUP_FILENAMES.getKey(), yum.getGroupFileNames());
                packageTypeConfig
                        .put(YUM_ROOT_DEPTH.getKey(),
                                String.valueOf(yum.getMetadataFolderDepth()));
                packageTypeConfig
                        .put(CALCULATE_YUM_METADATA.getKey(),
                                String.valueOf(yum.isAutoCalculateYumMetadata()));
                packageTypeConfig
                        .put(ENABLE_FILE_LISTS_INDEXING.getKey(), String.valueOf(yum.isEnableFileListsIndexing()));
            }
            case Docker -> {
                DockerTypeSpecificConfigModel docker = (DockerTypeSpecificConfigModel) type;
                packageTypeConfig.put(DOCKER_API_VERSION.getKey(), docker.getDockerApiVersion().toString());
                packageTypeConfig.put(MAX_UNIQUE_TAGS.getKey(), String.valueOf(docker.getMaxUniqueTags()));
                packageTypeConfig.put(BLOCK_PUSHING_SCHEMA_1.getKey(), String.valueOf(
                        DockerApiVersion.V2 == docker.getDockerApiVersion() && docker.getBlockPushingSchema1()));
                packageTypeConfig.put(DOCKER_TAG_RETENTION.getKey(), String.valueOf(docker.getDockerTagRetention()));
            }
            case Debian -> {
                DebTypeSpecificConfigModel deb = (DebTypeSpecificConfigModel) type;
                packageTypeConfig.put(DEBIAN_TRIVIAL_LAYOUT.getKey(), String.valueOf(deb.getTrivialLayout()));
                packageTypeConfig.put(OPTIONAL_INDEX_COMPRESSION_FORMATS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(deb.getOptionalIndexCompressionFormats()));
                packageTypeConfig.put(DDEB_SUPPORTED.getKey(), String.valueOf(deb.isDdebSupported()));
            }
            case NuGet -> {
                NugetTypeSpecificConfigModel nuget = (NugetTypeSpecificConfigModel) type;
                localConfig.getRepoTypeConfig().setMaxUniqueSnapshots(nuget.getMaxUniqueSnapshots());
            }
            case Cargo -> {
                CargoTypeSpecificConfigModel cargo = (CargoTypeSpecificConfigModel) type;
                packageTypeConfig.put(CARGO_ANONYMOUS_ACCESS.getKey(), String.valueOf(cargo.isAnonymousAccess()));
                packageTypeConfig.put(CARGO_INTERNAL_INDEX.getKey(), String.valueOf(cargo.isInternalIndex()));
            }
            case Composer -> {
                ComposerTypeSpecificConfigModel composer = (ComposerTypeSpecificConfigModel) type;
                packageTypeConfig.put(ENABLE_COMPOSER_V1_INDEXING.getKey(),
                        String.valueOf(composer.isEnableComposerV1Indexing()));
            }
            case Terraform -> {
                TerraformTypeSpecificConfigModel terraform = (TerraformTypeSpecificConfigModel) type;
                packageTypeConfig.put(TERRAFORM_TYPE.getKey(), terraform.getTerraformType());
            }
            default -> {
            }
        }
    }

    /**
     * Populates type specific values for remote repos
     */
    private void populateRemoteTypeSpecificDescriptorValues(TypeSpecificConfigModel type,
            RemoteRepoConfig remoteRepoConfig) {
        final Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        switch (type.getRepoType()) {
            case P2, Maven, Gradle, Ivy, SBT -> {
                MavenTypeSpecificConfigModel maven = (MavenTypeSpecificConfigModel) type;

                packageTypeConfig.put(FETCH_JARS_EAGERLY.getKey(), String.valueOf(maven.getEagerlyFetchJars()));
                packageTypeConfig.put(FETCH_SOURCES_EAGERLY.getKey(), String.valueOf(maven.getEagerlyFetchSources()));
                packageTypeConfig.put(REJECT_INVALID_JARS.getKey(), String.valueOf(maven.getRejectInvalidJars()));
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(maven.isListRemoteFolderItems());
                remoteRepoConfig.getRepoTypeConfig().setChecksumPolicyType(maven.getRemoteChecksumPolicy());

                //Set p2 url for supporting repos
                if (remoteRepoConfig.getRepoTypeConfig().getUrl() !=
                        null) { //Should always be true, but this is to avoid accidental nulls
                    packageTypeConfig.put(P2_ORIGINAL_URL.getKey(), remoteRepoConfig.getRepoTypeConfig().getUrl());
                }
            }
            case Bower -> {
                BowerTypeSpecificConfigModel bower = (BowerTypeSpecificConfigModel) type;
                buildAndSetBowerConfig(remoteRepoConfig, bower);
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(bower.isListRemoteFolderItems());
            }
            case CocoaPods -> {
                CocoaPodsTypeSpecificConfigModel pods = (CocoaPodsTypeSpecificConfigModel) type;
                buildAndSetPodsConfig(remoteRepoConfig, pods);
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(pods.isListRemoteFolderItems());

                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                        String.valueOf(pods.getEnableExternalDependencies()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(pods.getExternalPatterns()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(), null);
            }
            case VCS -> {
                VcsTypeSpecificConfigModel vcs = (VcsTypeSpecificConfigModel) type;
                buildAndSetVcsConfig(remoteRepoConfig, vcs);
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(vcs.isListRemoteFolderItems());
            }
            case Docker -> {
                DockerTypeSpecificConfigModel docker = (DockerTypeSpecificConfigModel) type;

                remoteRepoConfig.getRepoTypeConfig()
                        .setEnableTokenAuthentication(docker.getEnableTokenAuthentication());
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(docker.getListRemoteFolderItems());

                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                        String.valueOf(docker.getEnableForeignLayersCaching()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(docker.getExternalPatterns()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(), null); //not used
                packageTypeConfig.put(BLOCK_PUSHING_SCHEMA_1.getKey(), String.valueOf(
                        DockerApiVersion.V2 == docker.getDockerApiVersion() && docker.getBlockPushingSchema1()));

                remoteRepoConfig.getRepoTypeConfig().setRetrievalCachePeriodSecs(docker.getRetrievalCachePeriodSecs());
            }
            case Debian -> {
                DebTypeSpecificConfigModel deb = (DebTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(deb.isListRemoteFolderItems());

                packageTypeConfig.put(DDEB_SUPPORTED.getKey(), String.valueOf(deb.isDdebSupported()));
            }
            case NuGet -> {
                NugetTypeSpecificConfigModel nuGet = (NugetTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(nuGet.isListRemoteFolderItems());
                buildAndSetNugetConfig(remoteRepoConfig, nuGet);
            }
            case Generic -> {
                GenericTypeSpecificConfigModel generic = (GenericTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(generic.isListRemoteFolderItems());
            }
            case Go -> {
                GoTypeSpecificConfigModel go = (GoTypeSpecificConfigModel) type;
                buildAndSetGoConfig(remoteRepoConfig, go);
            }
            case YUM -> {
                YumTypeSpecificConfigModel yum = (YumTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(yum.isListRemoteFolderItems());
            }
            case Npm -> {
                NpmTypeSpecificConfigModel npm = (NpmTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(npm.isListRemoteFolderItems());
            }
            case Gems -> {
                GemsTypeSpecificConfigModel gems = (GemsTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(gems.isListRemoteFolderItems());
            }
            case Pypi -> {
                PypiTypeSpecificConfigModel pypi = (PypiTypeSpecificConfigModel) type;
                buildAndSetPyPIConfig(remoteRepoConfig, pypi);
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(pypi.isListRemoteFolderItems());
            }
            case GitLfs -> {
                GitLfsTypeSpecificConfigModel gitlfs = (GitLfsTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(gitlfs.isListRemoteFolderItems());
            }
            case Composer -> {
                ComposerTypeSpecificConfigModel composer = (ComposerTypeSpecificConfigModel) type;
                buildAndSetComposerConfig(remoteRepoConfig, composer);
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(composer.isListRemoteFolderItems());
            }
            case Chef -> {
                ChefTypeSpecificConfigModel chef = (ChefTypeSpecificConfigModel) type;
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(chef.isListRemoteFolderItems());
            }
            case Helm -> {
                HelmTypeSpecificConfigModel helm = (HelmTypeSpecificConfigModel) type;
                buildAndSetHelmConfig(remoteRepoConfig, helm);
            }
            case Cargo -> {
                CargoTypeSpecificConfigModel cargo = (CargoTypeSpecificConfigModel) type;
                buildAndSetCargoConfig(remoteRepoConfig, cargo);
            }
            case Terraform -> {
                TerraformTypeSpecificConfigModel terraform = (TerraformTypeSpecificConfigModel) type;
                buildAndSetTerraformRemoteConfig(remoteRepoConfig, terraform);
                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(terraform.isListRemoteFolderItems());
            }
            case Swift -> {
                SwiftTypeSpecificConfigModel swift = (SwiftTypeSpecificConfigModel) type;

                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                        String.valueOf(swift.getEnableExternalDependencies()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(swift.getExternalPatterns()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(), null); //not used

                remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(swift.getListRemoteFolderItems());
            }
            default -> {
            }
        }
    }

    private void populateVirtualTypeSpecific(TypeSpecificConfigModel type, VirtualRepoConfig virtualRepoConfig) {
        Map<String, String> packageTypeConfig = virtualRepoConfig.getPackageTypeConfig();
        switch (type.getRepoType()) {
            case P2:
                buildAndSetVirtualP2Config(virtualRepoConfig, (P2TypeSpecificConfigModel) type);
                break;
            case NuGet:
                NugetTypeSpecificConfigModel nuget = (NugetTypeSpecificConfigModel) type;
                packageTypeConfig.put(FORCE_NUGET_AUTHENTICATION.getKey(),
                        String.valueOf(nuget.isForceNugetAuthentication()));
                break;
            case Bower:
                BowerTypeSpecificConfigModel bower = (BowerTypeSpecificConfigModel) type;

                RemoteRepoConfig bowerRemoteRepoConfig =
                        repositoryConfigService.getRemoteRepoConfigByKey(bower.getExternalRemoteRepo());

                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                        String.valueOf(bower.getEnableExternalDependencies()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(bower.getExternalPatterns()));
                Optional.ofNullable(bowerRemoteRepoConfig).ifPresent(remoteRepoConfig ->
                        packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(),
                                remoteRepoConfig.getKey()));
                break;
            case Npm:
                NpmTypeSpecificConfigModel npm = (NpmTypeSpecificConfigModel) type;

                RemoteRepoConfig npmRemoteRepoConfig =
                        repositoryConfigService.getRemoteRepoConfigByKey(npm.getExternalRemoteRepo());

                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                        String.valueOf(npm.getEnableExternalDependencies()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(npm.getExternalPatterns()));
                Optional.ofNullable(npmRemoteRepoConfig).ifPresent(remoteRepoConfig ->
                        packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(),
                                remoteRepoConfig.getKey()));
                VirtualCacheConfig virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(npm.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Go:
                GoTypeSpecificConfigModel go = (GoTypeSpecificConfigModel) type;

                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                        String.valueOf(go.getEnableExternalDependencies()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(go.getExternalPatterns()));
                packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(), null); // Not used by go
                break;
            case YUM:
                YumTypeSpecificConfigModel yum = (YumTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(yum.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Helm:
                HelmTypeSpecificConfigModel helm = (HelmTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(helm.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);

                packageTypeConfig.put(USE_NAMESPACES.getKey(), String.valueOf(helm.getUseNamespaces()));
                break;
            case Alpine:
                AlpineTypeSpecificConfigModel alpine = (AlpineTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(alpine.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Cargo:
                CargoTypeSpecificConfigModel cargo = (CargoTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(cargo.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                packageTypeConfig.put(CARGO_INTERNAL_INDEX.getKey(), Boolean.TRUE.toString());
                break;
            case CRAN:
                CranTypeSpecificConfigModel cran = (CranTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(cran.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Conan:
                ConanTypeSpecificConfigModel conan = (ConanTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                packageTypeConfig.put(FORCE_CONAN_AUTHENTICATION.getKey(),
                        String.valueOf(conan.isForceConanAuthentication()));
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(conan.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Conda:
                CondaTypeSpecificConfigModel conda = (CondaTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(conda.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Chef:
                ChefTypeSpecificConfigModel chef = (ChefTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(chef.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            case Debian:
                DebTypeSpecificConfigModel debian = (DebTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(debian.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);

                packageTypeConfig.put(DEBIAN_DEFAULT_ARCHITECTURES.getKey(),
                        String.join(",", debian.getVirtualArchitectures()));
                packageTypeConfig.put(DEBIAN_OPTIONAL_INDEX_COMPRESSION_FORMATS.getKey(),
                        JsonUtils.getInstance().writeValueAsString(debian.getOptionalIndexCompressionFormats()));
                packageTypeConfig.put(DDEB_SUPPORTED.getKey(), String.valueOf(debian.isDdebSupported()));
                break;
            case Maven: //P2 reaches maven also
                MavenTypeSpecificConfigModel maven = (MavenTypeSpecificConfigModel) type;
                packageTypeConfig.put(FORCE_MAVEN_AUTHENTICATION.getKey(),
                        String.valueOf(maven.isForceMavenAuthentication()));
            case Gradle:
            case Ivy:
            case SBT:
                MavenTypeSpecificConfigModel mavenTypes = (MavenTypeSpecificConfigModel) type;
                virtualRepoConfig.getSecurityConfig().setKeyPair(mavenTypes.getKeyPair());
                packageTypeConfig.put(POM_REPOSITORY_REFERENCES_CLEANUP_POLICY.getKey(),
                        mavenTypes.getPomCleanupPolicy().getType());
                break;
            case Docker:
                DockerTypeSpecificConfigModel docker = (DockerTypeSpecificConfigModel) type;
                packageTypeConfig.put(RESOLVE_DOCKER_TAGS_BY_TIMESTAMP.getKey(),
                        String.valueOf(docker.getResolveDockerTagsByTimestamp()));
                packageTypeConfig.put(CACHING_LOCAL_FOREIGN_LAYERS_ENABLED.getKey(),
                        String.valueOf(docker.getCachingLocalForeignLayersEnabled()));
                break;
            case Composer:
                ComposerTypeSpecificConfigModel composer = (ComposerTypeSpecificConfigModel) type;
                virtualCacheConfig = virtualRepoConfig.getRepoTypeConfig().getVirtualCacheConfig();
                virtualCacheConfig = virtualCacheConfig != null ? virtualCacheConfig : new VirtualCacheConfig();
                virtualCacheConfig.setVirtualRetrievalCachePeriodSecs(composer.getVirtualRetrievalCachePeriodSecs());
                virtualRepoConfig.getRepoTypeConfig().setVirtualCacheConfig(virtualCacheConfig);
                break;
            default:
                break;

        }
    }

    private void buildAndSetNugetConfig(RemoteRepoConfig remoteRepoConfig, NugetTypeSpecificConfigModel nuGet) {
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(DOWNLOAD_CONTEXT_PATH.getKey(), nuGet.getDownloadContextPath());
        packageTypeConfig.put(FEED_CONTEXT_PATH.getKey(), nuGet.getFeedContextPath());
        packageTypeConfig.put(V3_FEED_URL.getKey(), nuGet.getV3FeedUrl());
        packageTypeConfig.put(SYMBOL_SERVER_URL.getKey(), nuGet.getSymbolServerUrl());
    }

    private void buildAndSetBowerConfig(RemoteRepoConfig remoteRepoConfig, BowerTypeSpecificConfigModel bower) {
        buildAndSetVcsConfig(remoteRepoConfig, bower);
        remoteRepoConfig.getPackageTypeConfig().put(BOWER_REGISTRY_URL.getKey(), bower.getRegistryUrl());
    }

    private void buildAndSetVcsConfig(RemoteRepoConfig remoteRepoConfig, VcsTypeSpecificConfigModel vcs) {
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(VCS_GIT_PROVIDER.getKey(), vcs.getGitProvider().name());
        packageTypeConfig.put(VCS_GIT_DOWNLOAD_URL.getKey(), vcs.getDownloadUrl());
        packageTypeConfig.put(VCS_TYPE.getKey(), vcs.getVcsType().name());
        remoteRepoConfig.getRepoTypeConfig().setMaxUniqueSnapshots(vcs.getMaxUniqueSnapshots());
        remoteRepoConfig.getRepoTypeConfig().setListRemoteFolderItems(vcs.isListRemoteFolderItems());
    }

    private void buildAndSetGoConfig(RemoteRepoConfig descriptor, GoTypeSpecificConfigModel go) {
        buildAndSetVcsConfig(descriptor, go);
    }

    private void buildAndSetPyPIConfig(RemoteRepoConfig remoteRepoConfig, PypiTypeSpecificConfigModel pyPI) {
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(PYPI_REGISTRY_URL.getKey(), pyPI.getRegistryUrl());
        packageTypeConfig.put(PYPI_REPOSITORY_SUFFIX.getKey(), pyPI.getRepositorySuffix());
    }

    private void buildAndSetComposerConfig(RemoteRepoConfig remoteRepoConfig,
            ComposerTypeSpecificConfigModel composer) {
        buildAndSetVcsConfig(remoteRepoConfig, composer);
        remoteRepoConfig.getPackageTypeConfig().put(COMPOSER_REGISTRY_URL.getKey(), composer.getRegistryUrl());
    }

    private void buildAndSetTerraformRemoteConfig(RemoteRepoConfig remoteRepoConfig,
            TerraformTypeSpecificConfigModel terraform) {
        buildAndSetVcsConfig(remoteRepoConfig, terraform);

        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(TERRAFORM_REGISTRY_URL.getKey(), terraform.getRegistryUrl());
        packageTypeConfig.put(TERRAFORM_PROVIDERS_URL.getKey(), terraform.getProvidersUrl());
    }

    private void buildAndSetPodsConfig(RemoteRepoConfig remoteRepoConfig, CocoaPodsTypeSpecificConfigModel pods) {
        buildAndSetVcsConfig(remoteRepoConfig, pods);
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();

        packageTypeConfig.put(COCOA_PODS_SPECS_URL.getKey(), pods.getSpecsRepoUrl());
        //Specs repo provider must be Artifactory or the same as the remote's Git provider - from UI it should always
        // match in any case, users can change it from the descriptor manually.
        packageTypeConfig.put(VCS_GIT_PROVIDER.getKey(), pods.getGitProvider().name());
    }

    private void buildAndSetVirtualP2Config(VirtualRepoConfig virtualRepoConfig, P2TypeSpecificConfigModel p2) {
        List<String> p2Repos = p2.getP2Repos().stream()
                .map(P2Repo::getRepoUrl)
                .distinct()
                .toList();

        //This just adds the selected urls - creating repos etc is done at the service
        Map<String, String> packageTypeConfig = virtualRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(P2_URLS.getKey(), JsonUtils.getInstance().writeValueAsString(p2Repos));
    }

    private void buildAndSetXrayConfig(RealRepoConfig realRepoConfig, LocalBasicRepositoryConfigModel basic) {
        XrayRepoConfig xrayConfig = null;
        XrayRepoConfigModel xrayConfigModel = basic.getXrayConfig();
        if (xrayConfigModel != null) {
            xrayConfig = new XrayRepoConfig();
            xrayConfig.setEnabled(xrayConfigModel.isEnabled());
            xrayConfig.setDataTtl(xrayConfigModel.getDataTtl());
        }
        realRepoConfig.getSecurityConfig().setXrayConfig(xrayConfig);
    }

    /**
     * Reason for entire model: currently only one field - 'enabled', but will expand in the near future
     */
    private void buildAndSetDownloadRedirectConfig(RealRepoConfig localConfig,
            LocalAdvancedRepositoryConfigModel advanced) {
        DownloadRedirectRepoConfig downloadRedirectConfig = null;
        DownloadRedirectRepoConfigModel downloadRedirectConfigModel = advanced.getDownloadRedirectConfig();
        if (downloadRedirectConfigModel != null && !AolUtils.isAol()) {
            downloadRedirectConfig = new DownloadRedirectRepoConfig();
            downloadRedirectConfig.setEnabled(downloadRedirectConfigModel.isEnabled());
        }
        localConfig.getRepoTypeConfig().setDownloadRedirectConfig(downloadRedirectConfig);
    }

    private void buildAndSetCdnRedirectConfig(RealRepoConfig realRepoConfig,
            RepositoryConfigModel<?, ?, ?> model) {
        CdnRedirectRepoConfigModel cdnRedirectConfigModel = model.getAdvanced().getCdnRedirectRepoConfig();
        if (cdnRedirectConfigModel != null) {
            CdnRedirectRepoConfig cdnRedirectConfig = new CdnRedirectRepoConfig();
            cdnRedirectConfig.setEnabled(cdnRedirectConfigModel.isEnabled());
            realRepoConfig.getRepoTypeConfig().setCdnRedirectRepoConfig(cdnRedirectConfig);
        }
    }

    private void buildAndSetHelmConfig(RemoteRepoConfig remoteRepoConfig, HelmTypeSpecificConfigModel helm) {
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(CHARTS_BASE_URL.getKey(), helm.getChartsBaseUrl());
        packageTypeConfig.put(EXTERNAL_DEPENDENCIES_ENABLED.getKey(),
                String.valueOf(helm.getEnableExternalDependencies()));
        packageTypeConfig.put(EXTERNAL_DEPENDENCIES_PATTERNS.getKey(),
                JsonUtils.getInstance().writeValueAsString(helm.getExternalPatterns()));
        packageTypeConfig.put(EXTERNAL_DEPENDENCIES_REMOTE_REPO_REF.getKey(), null);
    }

    private void buildAndSetCargoConfig(RemoteRepoConfig remoteRepoConfig, CargoTypeSpecificConfigModel cargo) {
        Map<String, String> packageTypeConfig = remoteRepoConfig.getPackageTypeConfig();
        packageTypeConfig.put(GIT_REGISTRY_URL.getKey(), cargo.getRegistryUrl());
        packageTypeConfig.put(CARGO_ANONYMOUS_ACCESS.getKey(), String.valueOf(cargo.isAnonymousAccess()));
        packageTypeConfig.put(CARGO_INTERNAL_INDEX.getKey(), String.valueOf(cargo.isInternalIndex()));
    }


    private PropertySet getPropSetByName(String name) {
        return centralConfig.getDescriptor().getPropertySets().stream()
                .filter(propertySet -> propertySet.getName().equals(name))
                .findFirst()
                .orElse(null);
    }
}
