package org.artifactory.ui.rest.resource.utils.repositories;

import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.exception.AuthorizationRestException;
import org.artifactory.ui.rest.service.utils.UtilsServiceFactory;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;

import javax.ws.rs.core.Response;
import java.util.stream.Stream;

import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

class PagedRepositoriesDataResourceTest {

    @Mock
    private UtilsServiceFactory utilsFactory;
    @Mock
    private AuthorizationService authorizationService;

    @Spy
    private PagedRepositoriesDataResource resource;
    private AutoCloseable mocks;

    @BeforeEach
    void setUp() {
        mocks = MockitoAnnotations.openMocks(this);
        resource.setUtilsFactory(utilsFactory);
        resource.setAuthorizationService(authorizationService);
        doReturn(mock(Response.class)).when(resource).runService(any());
    }

    @AfterEach
    void tearDown() throws Exception {
        mocks.close();
    }

    @Test
    void testSimpleUser() {
        setAuthorisationService(false, false);

        assertThrows(AuthorizationRestException.class, () -> {
            resource.getPagedRepoData();
        });
    }

    @ParameterizedTest
    @MethodSource("userBooleanProvider")
    void testAdminAndUserWithManageResourcesPermission(boolean isAdmin, boolean isResourceManager) {
        setAuthorisationService(isAdmin, isResourceManager);

        resource.getPagedRepoData();

        verify(resource, times(1)).runService(any());
    }

    private void setAuthorisationService(boolean isAdmin, boolean isResourceManager) {
        when(authorizationService.isAdmin()).thenReturn(isAdmin);
        when(authorizationService.isResourceManager()).thenReturn(isResourceManager);
    }

    private static Stream<Arguments> userBooleanProvider() {
        return Stream.of(
                Arguments.of(true, false),
                Arguments.of(false, true),
                Arguments.of(true, true)
        );
    }
}
